@@ -26,6 +26,7 @@ horizontalcall::horizontalcall(QWidget *parent) :
     existingLetterButtons = new QList<QPushButton*>();
     existingLetters = new QList<QString>();
     m_CurrentFilter = QString();
+    m_DisplayedPhoneNumbersButtons = QList<QPushButton*>();
 
     createSharedUI();
 
@@ -69,14 +70,19 @@ horizontalcall::horizontalcall(QWidget *parent) :
     m_scrollArea = NULL;
     m_FetchPending = true;
 
-    //DBus Related
-    //QDBusConnection::sessionBus().registerService(APP_DBUS_SERVICE);
-    //QDBusConnection::sessionBus().registerObject(APP_DBUS_PATH, this, QDBusConnection::ExportScriptableSlots);
+    QTimer::singleShot(1000, this, SLOT(fetchContacts()));
+}
+
+void horizontalcall::fetchContacts()
+{
+    lineEditFilter->setText("Loading...");
+    emit getContacts();
 }
 
 void horizontalcall::ContactUpdatesDone()
 {
     qDebug() << "Main Thread : Done updateing contacts.";
+    lineEditFilter->setText("");
     m_FetchPending = false;
     lineEditTextChanged(lineEditFilter->text());
 }
@@ -133,6 +139,13 @@ void horizontalcall::doAction2()
     }
 }
 
+void horizontalcall::doAction3()
+{
+    listWidgetFiltered->clear();
+    deleteLetters();
+    fetchContacts();
+}
+
 void horizontalcall::createSharedUI()
 {
     //Set mainLayout.
@@ -141,13 +154,16 @@ void horizontalcall::createSharedUI()
     //Create menu
     action1 = new QAction(tr("&AutoRotation"), this);
     action2 = new QAction(tr("&ManualRotation"), this);
+    action3 = new QAction(tr("&Refresh Contacts"), this);
 
     menu = menuBar()->addMenu(tr("Menu"));
     menu->addAction(action1);
     menu->addAction(action2);
+    menu->addAction(action3);
 
     connect(action1, SIGNAL(triggered()), this , SLOT(doAction1()));
     connect(action2, SIGNAL(triggered()), this , SLOT(doAction2()));
+    connect(action3, SIGNAL(triggered()), this , SLOT(doAction3()));
 
     //List Widget Filtered
     listWidgetFiltered = new QListWidget(centralWidget());
@@ -369,36 +385,24 @@ void horizontalcall::resizeEvent(QResizeEvent *e)
         lineEditFilter->setText(m_CurrentFilter);
         showContactDetails(m_CurrentRow);
 
+
     } else {
         lineEditFilter->setText(m_CurrentFilter);
         qDebug("Main Thread : Calling lineEdit Text Changed After UI Change");
         lineEditTextChanged(m_CurrentFilter);
     }
-
-    /*if (isFullScreen())
-    {
-        if (m_Landscape) {
-            this->resize(800,480);
-        } else {
-            this->resize(480,800);
-        }
-    } else {
-        if (m_Landscape) {
-            this->resize(800,424);
-        } else {
-            this->resize(480,744);
-        }
-    }*/
 }
 
 void horizontalcall::closeEvent(QCloseEvent *event)
 {
+    qDebug() << "Main Thread : Close event Called.";
     hide();
     event->ignore();
 }
 
 int horizontalcall::top_application()
 {
+    qDebug() << "Main Thread : Top Application Slot Called.";
     show();
     return 0;
 }
@@ -538,6 +542,7 @@ QPushButton:pressed {\
 
 void horizontalcall::clearButtonClicked()
 {
+    qDebug() << "Clear Button Clicked";
     //this->setUpdatesEnabled(false);
     if (m_scrollArea != NULL) {
         m_scrollArea->close();
@@ -615,6 +620,8 @@ void horizontalcall::createLetters()
     {
         CreateLetter(existingLetters->at(i));
     }
+
+    m_showingContactOptions = false;
 }
 
 void horizontalcall::deleteLetters()
@@ -630,6 +637,7 @@ void horizontalcall::deleteLetters()
 
 void horizontalcall::exitButtonClicked()
 {
+
     close();
 }
 
@@ -642,6 +650,7 @@ void horizontalcall::scanContacts()
 
         for (int j = 0; j < m_NameStrings.at(i).count(); j++)
         {
+            qDebug() << "Main Thread : Running scan contacts";
             if (m_NameStrings.at(i).at(j).length() <= lineEditFilter->text().length())
             {
                 continue;
@@ -719,10 +728,12 @@ void horizontalcall::showContactDetails(int row)
     qDebug("Main Thread : " + a.displayLabel().toAscii());
 
     QList<QContactPhoneNumber> allDetails = a.details<QContactPhoneNumber>();
+    m_DisplayedPhoneNumbersButtons.clear();
 
     for (int i = 0; i < allDetails.size(); i++) {
 
         QPushButton *aButton = new QPushButton(allDetails.at(i).number(), myWidget);
+        m_DisplayedPhoneNumbersButtons.append(aButton);
         aButton->sizePolicy().setVerticalPolicy(QSizePolicy::Expanding);
         if (aButton->text().length() > 11) {
             aButton->setStyleSheet(m_StylePhoneNumbersShort);
@@ -746,13 +757,37 @@ void horizontalcall::showContactDetails(int row)
 
 void horizontalcall::phoneNumberClicked()
 {
-    QMessageBox msgBox;
-    QPushButton *callButton = msgBox.addButton(tr("Call"), QMessageBox::YesRole);
-    QPushButton *smsButton = msgBox.addButton(tr("SMS"), QMessageBox::NoRole);
+    qDebug() << "Phone number button clicked";
+    QPushButton *tempButton = qobject_cast<QPushButton *>(QObject::sender());
+    m_Phonenumber = tempButton->text();
 
-    msgBox.exec();
+    /*for (int i = 1; i < m_scrollArea->widget()->layout()->count();i++) {
+        qDebug() << "Deleting widget";
+        //QLayoutItem *layoutItemToDelete = m_scrollArea->widget()->layout()->itemAt(i);
+        m_scrollArea->widget()->layout()->removeItem(m_scrollArea->widget()->layout()->itemAt(i));
+    }*/
+    for (int i = 0; i < m_DisplayedPhoneNumbersButtons.count(); i++)
+    {
+        delete m_DisplayedPhoneNumbersButtons.at(i);
+    }
 
-    if (msgBox.clickedButton() == callButton) {
+    callNumberButton = new QPushButton(m_scrollArea->widget());
+    sendSMSButton = new QPushButton(m_scrollArea->widget());
+    callNumberButton->setText("Call");
+    callNumberButton->setStyleSheet(m_StylePhoneNumbers);
+    callNumberButton->sizePolicy().setVerticalPolicy(QSizePolicy::Expanding);
+    sendSMSButton->setText("SMS");
+    sendSMSButton->setStyleSheet(m_StylePhoneNumbers);
+    sendSMSButton->sizePolicy().setVerticalPolicy(QSizePolicy::Expanding);
+    m_scrollArea->widget()->layout()->addWidget(callNumberButton);
+    m_scrollArea->widget()->layout()->addWidget(sendSMSButton);
+
+    connect(callNumberButton, SIGNAL(clicked()),this, SLOT(callNumber()));
+    connect(sendSMSButton, SIGNAL(clicked()),this, SLOT(sendSMS()));
+
+    m_showingContactOptions = true;
+
+    /*if (msgBox.clickedButton() == callButton) {
         qDebug() << "Main Thread : Call";
         QPushButton *tempButton = qobject_cast<QPushButton *>(QObject::sender());
         m_CallButtons.append(tempButton);
@@ -774,7 +809,31 @@ void horizontalcall::phoneNumberClicked()
         tempButton->setEnabled(false);
         //makeDbusGSMCall(temp);
         sendFastSMS(temp);
-    }
+    }*/
+}
+
+void horizontalcall::sendSMS()
+{
+    qDebug() << "Main Thread : SMS";
+    m_CallButtons.append(sendSMSButton);
+    QString temp = sendSMSButton->text();
+    m_PhoneNumbers.append(new QString("SMS"));
+    sendSMSButton->setText("Sending sms...");
+    QTimer::singleShot(500, this, SLOT(enableButtons()));
+    sendSMSButton->setEnabled(false);
+    sendFastSMS(m_Phonenumber);
+}
+
+void horizontalcall::callNumber()
+{
+    qDebug() << "Main Thread : Call";
+    m_CallButtons.append(callNumberButton);
+    QString temp = callNumberButton->text();
+    m_PhoneNumbers.append(new QString("Call"));
+    callNumberButton->setText("Dialing...");
+    QTimer::singleShot(500, this, SLOT(enableButtons()));
+    callNumberButton->setEnabled(false);
+    makeDbusGSMCall(m_Phonenumber);
 }
 
 void horizontalcall::sendFastSMS(QString number)
@@ -806,6 +865,7 @@ void horizontalcall::sendFastSMS(QString number)
 
 void horizontalcall::enableButtons()
 {
+    qDebug() << "Enabling Buttons";
     m_CallButtons.at(0)->setEnabled(true);
     m_CallButtons.at(0)->setText(*m_PhoneNumbers.at(0));
     m_CallButtons.removeFirst();
@@ -814,7 +874,7 @@ void horizontalcall::enableButtons()
 
 void horizontalcall::itemClicked(QModelIndex model)
 {
-    //qDebug("Main Thread : Row Clicked : " + QString::number(model.row()).toAscii());
+    qDebug() << "Main Thread : Row Clicked";
     showContactDetails(model.row());
     m_CurrentRow = model.row();
 }
