@@ -39,15 +39,21 @@ public slots:
     void minimizeButtonClicked();
     void doAction1();
     void doAction2();
+    void doAction3();
     void AddContact(QtMobility::QContact temp);
     void ContactUpdatesDone();
     void Copy_m_Contacts(QList<QtMobility::QContact>);
+    void fetchContacts();
+    void callNumber();
+    void sendSMS();
     Q_SCRIPTABLE int top_application();
 
 signals:
+    void getContacts();
 
 public:
     explicit horizontalcall(QWidget *parent = 0);
+    QThread *m_myThread;
     ~horizontalcall();
 
 protected:
@@ -87,6 +93,11 @@ private:
     bool m_FetchPending;
     bool m_DisplayingContact;
     bool m_AutorotationEnabled;
+    bool m_showingContactOptions;
+    QList<QPushButton*> m_DisplayedPhoneNumbersButtons;
+    QString m_Phonenumber;
+    QPushButton *callNumberButton;
+    QPushButton *sendSMSButton;
 
     //----------------- Shared UI Members -----------------
     //Layouts / Container Widgets
@@ -104,6 +115,7 @@ private:
     QLabel      *labelSelector;
     QAction     *action1;
     QAction     *action2;
+    QAction     *action3;
     QMenu       *menu;
     //--------------------------------------------------------
 
